#include "glider.h"
#include "graphics.h"
#include "resources.h"

GLIDER::GLIDER()
{
  for(int i = 0; i < 3; i++)
  {
    direction[i] = 0.0f;
    velocity[i] = 0.0f;
    position[i] = 0.0f;
  }
  
  direction.x = 0.0f;
  direction.y = 0.0f;
  direction.z = 0.0f;
}

void GLIDER::TurnLeft(bool yes)
{
  if(yes)
  {
    turning = true;
    direction.z += 0.4f * frame_time;
    if(direction.z >= 0.125f)
    {
      direction.z = 0.125f;
    }
  }
  else
  {
    turning = false;
    if(direction.z > 0.0f)
    {
      direction.z -= 0.2f * frame_time;
      if(direction.z < 0.0f)
      {
        direction.z = 0.0f;
      }
    }
  }
}

void GLIDER::TurnRight(bool yes)
{
  if(yes)
  {
    turning = true;
    direction.z -= 0.4f * frame_time;
    if(direction.z <= -0.125f)
    {
      direction.z = -0.125f;
    }
  }
  else
  {
    turning = false;
    if(direction.z < 0.0f)
    {
      direction.z += 0.2f * frame_time;
      if(direction.z > 0.0f)
      {
        direction.z = 0.0f;
      }
    }
  }
}

void GLIDER::UpdateDirection(void)
{
  direction.y += direction.z * 2 * frame_time;
  
  while(direction.y > 1.0f)
  {
    direction.y--;
  }
  while(direction.y < 0.0f)
  {
    direction.y++;
  }
}

void GLIDER::TiltUp(bool yes)
{
  if(yes)
  {
    direction.x += 0.3f * frame_time;
    if(direction.x > 0.125f)
    {
      direction.x = 0.125f;
    }
  }
  else
  {
    if(direction.x > 0.0f)
    {
      direction.x -= 0.2f * frame_time;
      if(direction.x < 0.0f)
      {
        direction.x = 0.0f;
      }
    }
  }
}

void GLIDER::TiltDown(bool yes)
{
  if(yes)
  {
    direction.x -= 0.3f * frame_time;
    if(direction.x < -0.125f)
    {
      direction.x = -0.125f;
    }
  }
  else
  {
    if(direction.x < 0.0f)
    {
      direction.x += 0.2f * frame_time;
      if(direction.x > 0.0f)
      {
        direction.x = 0.0f;
      }
    }
  }
}

void GLIDER::ThrusterOn(bool yes)
{
  VECTOR3D thrust;
  VECTOR3D friction;
  
  if(yes)
  {
    thrust.z = cos(direction.y * 2.0f * 3.14159f);
    thrust.y = direction.x * -8.0f;
    thrust.x = sin(direction.y * 2.0f * 3.14159f);
    engine_intensity += 1.5 * frame_time;
    if(engine_intensity >= 0.85f)
    {
      engine_intensity = 0.85f;
    }
  }
  else
  {
    thrust.x = 0.0f;
    thrust.y = 0.0f;
    thrust.z = 0.0f;
  }
  
  velocity.x += thrust.x * 160.0f * frame_time;
  velocity.y += thrust.y * 160.0f * frame_time;
  velocity.z += thrust.z * 160.0f * frame_time;
  
  friction.x = velocity.x / 1.5f;
  friction.y = velocity.y / 1.5f;
  friction.z = velocity.z / 1.5f;
  
  //position.x -= velocity.x * frame_time;
  //position.y -= velocity.y * frame_time;
  //position.z -= velocity.z * frame_time;
  
  velocity.x -= friction.x * frame_time;
  velocity.y -= friction.y * frame_time;
  velocity.z -= friction.z * frame_time;
  
  engine_intensity -= 0.3f * frame_time;
}

void GLIDER::UseBoost(void)
{
  VECTOR3D thrust;
  
  thrust.z = cos(direction.y * 2.0f * 3.14159f);
  thrust.y = direction.x * -8.0f;
  thrust.x = sin(direction.y * 2.0f * 3.14159f);
  
  velocity.x = thrust.x * 300.0f;
  velocity.y = thrust.y * 180.0f;
  velocity.z = thrust.z * 300.0f;
  
  engine_intensity = 1.0f;
}

void GLIDER::GetCameraPosition(VECTOR3D &camera)
{
  float x_vec;
  float y_vec;
  
  //Break the y rotation up into x and y vectors.
  //Goes from -1 to 1. 
  
  x_vec = cos(direction.y * 2.0f * 3.14159f);
  y_vec = sin(direction.y * 2.0f * 3.14159f);
  
  camera.x = position.x + y_vec * 8;
  camera.y = position.y - direction.x * 26.0f;
  camera.z = position.z + x_vec * 8;
}

void GLIDER::GetNextPosition(VECTOR3D &next_pos)
{
  next_pos.x = position.x - velocity.x * frame_time;
  next_pos.y = position.y - velocity.y * frame_time;
  next_pos.z = position.z - velocity.z * frame_time;
}

void GLIDER::SetPosition(VECTOR3D &pos)
{
  position.x = pos.x;
  position.y = pos.y;
  position.z = pos.z;
}

void GLIDER::GetPosition(VECTOR3D &pos)
{
  pos = position;
}

void GLIDER::SetVelocity(VECTOR3D &vel)
{
  velocity = vel;
}

float GLIDER::GetSpeed(void)
{
  return velocity.GetLength();
}

void GLIDER::ResetVelocity(void)
{
  velocity.x = 0.0f;
  velocity.y = 0.0f;
  velocity.z = 0.0f;
}

void GLIDER::ResetYVelocity(void)
{
  velocity.y = 0.0f;
}

void GLIDER::SetDirection(VECTOR3D &dir)
{
  direction = dir;
}

void GLIDER::Draw(void)
{
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);
  glTranslatef(position.x, position.y, position.z);
  glRotatef(360.0f * direction.y, 0.0f, 0.1f, 0.0f);
  glRotatef(360.0f * direction.z, 0.0f, 0.0f, 1.0f);
  glRotatef(360.0f * direction.x, 1.0f, 0.0f, 0.0f);
  
  glider_mesh.Render();
  
  //Now draw the engine effects.
  
  glDisable(GL_CULL_FACE);
  glDisable(GL_LIGHTING);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
  glDepthMask(GL_FALSE);
  glEnable(GL_TEXTURE_2D);
  
  glPushMatrix();
  
  glTranslatef(-0.37f, -0.4f, 0.0f);
  glScalef(2.0f, 2.0f, 1.0f);
  light_texture.Bind();
  
  glBegin(GL_QUADS);
  
  glColor4f(1.0f, 0.5f, 0.5f, engine_intensity);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-0.2f, 0.2f, 0.87f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(-0.20f, -0.20f, 0.87f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(0.2f, -0.20f, 0.87f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(0.2f, 0.2f, 0.87f);
  
  glEnd();
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(0.37f, -0.4f, 0.0f);
  glScalef(2.0f, 2.0f, 1.0f);
  light_texture.Bind();
  
  glBegin(GL_QUADS);
  
  glColor4f(1.0f, 0.5f, 0.5f, engine_intensity);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-0.2f, 0.2f, 0.87f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(-0.20f, -0.20f, 0.87f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(0.2f, -0.20f, 0.87f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(0.2f, 0.2f, 0.87f);
  
  glEnd();
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(-1.9f, -0.6f, 0.0f);
  glScalef(1.0f, 1.0f, 1.0f);
  light_texture.Bind();
  
  glBegin(GL_QUADS);
  
  glColor4f(1.0f, 0.5f, 0.5f, engine_intensity);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-0.2f, 0.2f, 0.8f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(-0.20f, -0.20f, 0.8f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(0.2f, -0.20f, 0.8f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(0.2f, 0.2f, 0.8f);
  
  glEnd();
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(1.9f, -0.6f, 0.0f);
  glScalef(1.0f, 1.0f, 1.0f);
  light_texture.Bind();
  
  glBegin(GL_QUADS);
  
  glColor4f(1.0f, 0.5f, 0.5f, engine_intensity);
  glTexCoord2f(0.0f, 1.0f);
  glVertex3f(-0.2f, 0.2f, 0.8f);
  glTexCoord2f(0.0f, 0.0f);
  glVertex3f(-0.20f, -0.20f, 0.8f);
  glTexCoord2f(1.0f, 0.0f);
  glVertex3f(0.2f, -0.20f, 0.8f);
  glTexCoord2f(1.0f, 1.0f);
  glVertex3f(0.2f, 0.2f, 0.8f);
  
  glEnd();
  
  glPopMatrix();
  
  glEnable(GL_CULL_FACE);
  glEnable(GL_LIGHTING);
  glDisable(GL_BLEND);
  glDepthMask(GL_TRUE);
  
  glPopMatrix();
}
