#include <OpenLayer.hpp>
#include <GL/glu.h>
#include "graphics.h"

float frame_time;

void InitGraphics(void)
{
  bool fullscreen;
  
  std::ifstream full("fullscreen.txt");
  
  full >> fullscreen;
  
  ol::Settings::SetAntialiasing(false);
  ol::Setup::SetupProgram();
  ol::Settings::SetAntialiasing(false);
  ol::Setup::SetupScreen(800, 600, fullscreen, 32, 8);
  set_window_title("G-Loop"); 

  ol::FpsCounter::Start(1.0f);
  frame_time = 0.0f;
  
  ClearScreen();
}

void Init2DDrawingMode(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //glFrustum(-1.0, 1.0, -0.75, 0.75, 1.0, 1024.0);
  ol::Settings::SetOrthographicProjection(800, 600);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
  
  glDisable(GL_FOG);
  
  glDisable(GL_CULL_FACE);
  glColor3f(1.0f, 1.0f, 1.0f);
  glDisable(GL_DEPTH_TEST);	
  glShadeModel(GL_SMOOTH);	
  
  glDisable(GL_LIGHT0);
  glDisable(GL_LIGHTING);
  
  glDisable(GL_NORMALIZE);
  //glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void Init3DDrawingMode(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //glFrustum(-1.0, 1.0, -0.75, 0.75, 1.0, 1024.0);
  gluPerspective(90.0f, 1.3f, 1.0f, 2200.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
  
  glEnable(GL_FOG);
  float fog_color[4] = {1.0f, 1.0f, 1.0f, 0.5f};
  glFogf(GL_FOG_MODE, GL_EXP);
  glFogf(GL_FOG_DENSITY, 0.0009f);
  glFogf(GL_FOG_START, 0.0f);
  glFogf(GL_FOG_END, 2000.0f);
  glFogfv(GL_FOG_COLOR, fog_color);
  
  glEnable(GL_CULL_FACE);
  glColor3f(1.0f, 1.0f, 1.0f);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);		
  glShadeModel(GL_SMOOTH);	
  
  float color_material[] = { 0.9f, 0.9f, 0.9f, 1 };
  glEnable(GL_LIGHT0);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, color_material);
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, color_material);
  glEnable(GL_LIGHTING);
  
  glEnable(GL_NORMALIZE);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void ClearScreen(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
}

void FlipScreen(void)
{
  ol::Canvas::Refresh();
  frame_time = ol::FpsCounter::NewFrameStarted(); 
}
