#include <OpenLayer.hpp>
#include "heightmap.h"

HEIGHTMAP::HEIGHTMAP()
{
  heightmap_data = NULL;
  heightmap_vertex = NULL;
  heightmap_vertex_num = 0;
  
  x_pos = 0.0f;
  y_pos = 0.0f;
  z_pos = 0.0f; 
  min_y = -400.0f; 
  max_y = 400.0f; 
  pixel_size = 40.0f;
  
  display_list = 0;
}

HEIGHTMAP::~HEIGHTMAP()
{
  Destroy();
}

void HEIGHTMAP::LoadBitmap(const char *filename)
{
  Destroy();
  
  heightmap_data = load_bitmap(filename, NULL);
}

void HEIGHTMAP::Destroy(void)
{
  if(heightmap_data != NULL)
  {
    destroy_bitmap(heightmap_data);
    heightmap_data = NULL;
  }
  
  if(heightmap_vertex != NULL)
  {
    delete[] heightmap_vertex;
    heightmap_vertex = NULL;
  }
  
  if(display_list != 0)
  {
    glDeleteLists(display_list, 1);
    display_list = 0;
  }
  
  heightmap_vertex_num = 0;
}

void HEIGHTMAP::SetCenterXYZ(float x, float y, float z)
{
  x_pos = x; y_pos = y; z_pos = z;
}

void HEIGHTMAP::SetPixelSize(float pix_size)
{
  pixel_size = pix_size;
}

void HEIGHTMAP::SetMinMaxHeight(float minimum_y, float maximum_y)
{
  min_y = minimum_y; max_y = maximum_y;
}

void HEIGHTMAP::GenerateHeightmapVertexData(void)
{
  //std::ofstream output("debugging.txt");
  int pixel_x, pixel_y, pixel_value;
  int cur_pixel;
  float start_x, cur_x, cur_y, cur_z;
  float y_values;
  
  if(heightmap_data == NULL)
  {
    return;
  }
  
  if(heightmap_vertex != NULL)
  {
    delete[] heightmap_vertex;
  }
  
  y_values = max_y - min_y;
  
  cur_x = ((float)(heightmap_data->w) / -2.0f) * pixel_size;
  start_x = cur_x;
  cur_z = ((float)(heightmap_data->h) / -2.0f) * pixel_size;
  
  heightmap_vertex_num = heightmap_data->w * heightmap_data->h;
  heightmap_vertex = new float[heightmap_vertex_num * 3];
  
  cur_pixel = 0;
  for(pixel_y = 0; pixel_y < heightmap_data->h; pixel_y++)
  {
    for(pixel_x = 0; pixel_x < heightmap_data->w; pixel_x++)
    {
      pixel_value = getpixel(heightmap_data, pixel_x, pixel_y);
      pixel_value = getr(pixel_value);
      heightmap_vertex[cur_pixel] = cur_x;
      heightmap_vertex[cur_pixel + 1] = y_values * ((float)(pixel_value) / 255.0f) + min_y;
      heightmap_vertex[cur_pixel + 2] = cur_z;
      //output << heightmap_vertex[cur_pixel] << " ";
      //output << heightmap_vertex[cur_pixel + 1] << " ";
      //output << heightmap_vertex[cur_pixel + 2] << " ";
      cur_pixel+=3;
      cur_x += pixel_size;
    }
    //output << "\n";
    cur_x = start_x;
    cur_z += pixel_size;
  }
  
  if(display_list != 0)
  {
    glDeleteLists(display_list, 1);
    display_list = 0;
  }
  
  display_list = glGenLists(1);
  glNewList(display_list,GL_COMPILE);
  SendGLCommands();
  glEndList();
}

void HEIGHTMAP::SendGLCommands(void)
{
  //std::ofstream output("debugging_b.txt");
  
  int x, y;
  float tex_u, tex_v, tex_w;
  
  tex_u = 0.0f;
  tex_v = 0.0f;
  tex_w = 0.0f;
  float s_tex_w;
  
  for(y = 0; y < heightmap_data->h - 1; y++)
  {
    glBegin(GL_TRIANGLE_STRIP);
    for(x = 0; x < heightmap_data->w - 1; x++)
    {
      float x20 = heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3] - heightmap_vertex[y * heightmap_data->w * 3 + x * 3];
      float y20 = heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 1] - heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 1];
      float z20 = heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 2] - heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 2];
      float x13 = heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3] - heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3];
      float y13 = heightmap_vertex[(y + 1) * heightmap_data->w * 3 + (x) * 3 + 1] - heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 1];
      float z13 = heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3 + 2] - heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 2];

      /* Compute the cross product */
      float cpx = (z20 * y13) - (y20 * z13);
      float cpy = (x20 * z13) - (z20 * x13);
      float cpz = (y20 * x13) - (x20 * y13);
    
      glNormal3f(cpx, cpy, cpz);
      
      glTexCoord3f(tex_u, tex_w, (heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 1] - min_y) / (max_y - min_y));
      glVertex3f(heightmap_vertex[y * heightmap_data->w * 3 + x * 3],	heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 1], heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 2]);    
      
      s_tex_w = tex_w + 0.4f;
      
      glTexCoord3f(tex_u, s_tex_w, (heightmap_vertex[(y + 1) * heightmap_data->w * 3 + (x) * 3 + 1] - min_y) / (max_y - min_y));
      glVertex3f(heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3],	heightmap_vertex[(y + 1) * heightmap_data->w * 3 + (x) * 3 + 1], heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3 + 2]); 
      
      //glTexCoord3f(tex_u + 0.01f, tex_w, (heightmap_vertex[(y) * heightmap_data->w * 3 + (x + 1) * 3 + 1] - min_y) / (max_y - min_y));
      //glVertex3f(heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3],	heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 1], heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 2]);
      
      //glTexCoord3f(tex_u + 0.01f, tex_w + 0.01f, (heightmap_vertex[(y + 1) * heightmap_data->w * 3 + (x + 1) * 3 + 1] - min_y) / (max_y - min_y));
      //glVertex3f(heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3],	heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 1], heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 2]);
  
      tex_u += 0.4f;
  
      //output << heightmap_vertex[y * heightmap_data->w * 3 + x * 3] << " " <<	heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 1] << " " << heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 2] << "\n";
      //output << heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3] << " " <<	heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 1] << " " << heightmap_vertex[y * heightmap_data->w * 3 + (x+1) * 3 + 2] << "\n";
      //output << heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3] << " " <<	heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 1] << " " <<	 heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x+1) * 3 + 2] << "\n";
      //output << heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3] << " " <<	heightmap_vertex[(y) * heightmap_data->w * 3 + (x) * 3 + 1] << " " <<	 heightmap_vertex[(y+1) * heightmap_data->w * 3 + (x) * 3 + 2] << "\n\n";
    }
    glEnd();
    tex_u = 0.0f;
    tex_w += 0.4f;
  }
}

void HEIGHTMAP::Draw(void)
{
  glCallList(display_list);
}

float HEIGHTMAP::CollisionHeight(float x_pos, float z_pos)
{
  //std::ofstream file("tester.txt");
  int x, y;
  
  //Convert the x / z into coordinates.
  //file << x_pos << " " << z_pos << "\n";
  x_pos /= pixel_size;
  z_pos /= pixel_size;
  
  x_pos = x_pos + (float)(heightmap_data->w) / 2.0f;
  z_pos = z_pos + (float)(heightmap_data->h) / 2.0f;
  
  //file << x_pos << " " << z_pos << "\n";
  
  x = (int)x_pos;
  y = (int)z_pos;
  
  //file << x << " " << y << "\n";
  
  if(x < 0 || y < 0)
  {
    return min_y;
  }
  
  if(x >= heightmap_data->w || y >= heightmap_data->h)
  {
    return min_y;
  }
  
  return heightmap_vertex[y * heightmap_data->w * 3 + x * 3 + 1];
}
