#include "level_select_loop.h"
#include "input.h"
#include "graphics.h"
#include "resources.h"
#include <GL/glut.h>

void LEVEL_SELECT_LOOP::Init(void)
{
  title_font.Load("data/font.ttf", 60, 60, ol::Rgba::WHITE, 0);
  selection_font.Load("data/font.ttf", 32, 32, ol::Rgba::WHITE, 0);
  LoadLevels();
  cur_level = 0;
  bg_rotation = 0.0f;
  select_rotation = 0.0f;
  go_back = false;
}

bool LEVEL_SELECT_LOOP::Logic(void)
{
  char ttb_text[100];
  
  Init3DDrawingMode();
  
  glPushMatrix();
  
  glDisable(GL_TEXTURE_2D);
  glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
  
  glTranslatef(-20.0f, 0.0f, -20.0f);
  glRotatef(bg_rotation, 0.0f, 1.0f, 0.0f);
  glRotatef(90.0, 1.0f, 0.0f, 0.0f);
  
  glutWireSphere(30.0f, 40, 40);
  
  bg_rotation += 30.0f * frame_time;
  
  glPopMatrix();
  
  glPushMatrix();
  
  glEnable(GL_TEXTURE_2D);
  glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
  
  glTranslatef(0.0f, -1.0f, -5.0f);
  glRotatef(bg_rotation * 2.0, 0.0f, 1.0f, 0.0f);
  
  glider_mesh.Render();
  
  glPopMatrix();
  
  glEnable(GL_TEXTURE_2D);
  //glDisable(GL_LIGHTING);
  glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glEnable(GL_TEXTURE_GEN_S); 
  glEnable(GL_TEXTURE_GEN_T);
  
  glPushMatrix();
  
  glTranslatef(-10.0f, 5.5f, -10.0f);
  glRotatef(select_rotation, 1.0f, 0.0f, 0.0f);
  
  select_mesh.Render();
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(10.0f, 5.5f, -10.0f);
  glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
  glRotatef(-1.0f * select_rotation, 1.0f, 0.0f, 0.0f);
  
  select_mesh.Render();
  
  glPopMatrix();
  
  glDisable(GL_TEXTURE_GEN_S); 
  glDisable(GL_TEXTURE_GEN_T);
  
  select_rotation += 90.0f * frame_time;
  
  Init2DDrawingMode();
  
  title_font.Print("Level Select", 400 - title_font.FirstLineWidth("Level Select") / 2, 75);
  selection_font.Print(levels[cur_level].name, 400 - selection_font.FirstLineWidth(levels[cur_level].name) / 2, 150);
  
  sprintf(ttb_text, "Time To Beat: %i:%i:%i", times[cur_level].min, times[cur_level].sec, times[cur_level].ms);
  selection_font.Print(ttb_text, 400 - selection_font.FirstLineWidth(ttb_text) / 2, 250);
  
  if(KeyPress(KEY_RIGHT) || KeyPress(KEY_DOWN) || KeyPress(KEY_D) || KeyPress(KEY_S))
  {
    cur_level++;
    if(cur_level >= num_levels)
    {
      cur_level = num_levels - 1;
    }
    click_sound.Play();
  }
  
  if(KeyPress(KEY_LEFT) || KeyPress(KEY_UP) || KeyPress(KEY_A) || KeyPress(KEY_W))
  {
    cur_level--;
    if(cur_level < 0)
    {
      cur_level = 0;
    }
    click_sound.Play();
  }
  
  if(KeyPress(KEY_ESC) || KeyPress(KEY_ENTER) || KeyPress(KEY_LCONTROL))
  {
    if(KeyDown(KEY_ESC))
    {
      go_back = true;
    }
    else
    {
      select_sound.Play();
      strcpy(filename_to_load, levels[cur_level].filename);
      ms_to_beat = times[cur_level].min * 60 + times[cur_level].sec;
      ms_to_beat = ms_to_beat * 1000 + times[cur_level].ms;
    }
    return false;
  }
  return true;
}

bool LEVEL_SELECT_LOOP::CancelLevelLoad(void)
{
  return go_back;
}

void LEVEL_SELECT_LOOP::Shutdown(void)
{
  title_font.Destroy();
  selection_font.Destroy();
  
  levels.clear();
  times.clear();
}

void LEVEL_SELECT_LOOP::LoadLevels(void)
{
  std::ifstream file("data/levels.dat");
  LEVEL_DATA data;
  TIME_DATA time_data;
  
  file >> num_levels;
  
  file.getline(data.name, 60);
  
  for(int i = 0; i < num_levels; i++)
  {
    file.getline(data.name, 60);
    file.getline(data.filename, 60);
    levels.push_back(data);
  }
  
  for(int i = 0; i < num_levels; i++)
  {
    file >> time_data.min;
    file >> time_data.sec;
    file >> time_data.ms;
    
    times.push_back(time_data);
  }
}

char *LEVEL_SELECT_LOOP::GetFilenameToLoad(void)
{
  return filename_to_load;
}

int LEVEL_SELECT_LOOP::GetTTB(void)
{
  return ms_to_beat;
}
