#include <OpenLayer.hpp>
#include "heightmap.h"
#include "music.h"
#include "graphics.h"
#include "texture3d.h"
#include "texture.h"
#include "resources.h"
#include "input.h"
#include "sound.h"
#include "main_game_loop.h"
#include "tins_loop.h"
#include "title_loop.h"
#include "level_select_loop.h"
#include "stage_clear_loop.h"

void init();
void RunGame();
void deinit();

int main() 
{
  MUSIC music;
 	init();	

  music.LoadMidi("data/race.mid");
  music.Play();

  RunGame();

	deinit();
	return 0;
}
END_OF_MAIN()

void RunGame()
{
  MUSIC title_music;
  
  MAIN_GAME_LOOP game_loop;
  TINS_LOOP tins_loop;
  TITLE_LOOP title_loop;
  LEVEL_SELECT_LOOP level_loop;
  STAGE_CLEAR_LOOP clear_loop;
  int state = 1;
  
  title_music.LoadMidi("data/title.mid");
  
  title_music.Play();
  
  tins_loop.DoScene();
  
  do
  {
    switch(state)
    {
      case 1:
        title_loop.DoScene();
        if(title_loop.SelectQuit())
        {
          state = 0;
        }
        else
        {
          state = 2;
        }
        break;
      case 2:
        level_loop.DoScene();
        if(level_loop.CancelLevelLoad())
        {
          state = 1;
        }
        else
        {
          state = 3;
          title_music.Stop();
          
          game_loop.SetTTB(level_loop.GetTTB());
          game_loop.SetLevelFilename(level_loop.GetFilenameToLoad());
        }
        break;
      case 3:
        game_loop.DoScene();
        if(game_loop.LevelComplete())
        {
          clear_loop.SetTTB(level_loop.GetTTB());
          clear_loop.SetActualTime(game_loop.GetFinalTime());
          state = 4;
        }
        else
        {
          title_music.Stop();
          title_music.Play();
          state = 2;
        }
        break;    
      case 4:
        clear_loop.DoScene();
        title_music.Stop();
        title_music.Play();
        state = 2;
        break;
      default:
        break;
    }
  } while(state != 0);
}

void init() 
{
  InitGraphics();
  
  install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);
  set_volume(255, 140);
  
  LoadResources();
}

void deinit() 
{
  DestroyResources();
	clear_keybuf();
	/* add other deinitializations here */
}
