#include <OpenLayer.hpp>
#include <GL/glu.h>
#include <GL/glut.h>
#include <stdio.h>
#include "main_game_loop.h"
#include "input.h"
#include "graphics.h"
#include "resources.h"
#include "vector3d.h"
#include "ring.h"

RING test_ring;

void MAIN_GAME_LOOP::Init(void)
{
  VECTOR3D player_start;
  VECTOR3D direction_start;
  std::ifstream level_file(level_filename);
  int num_loops;
  int laps;
  char filenames[6][60];
  
  for(int i = 0; i < 6; i++)
  {
    level_file.getline(filenames[i], 60);
  } 
  
  hmap.LoadBitmap(filenames[0]); 
  hmap.GenerateHeightmapVertexData();
  
  tex3d.Load(filenames[1], filenames[2], filenames[3], filenames[4]);
  
  bgm.LoadMidi(filenames[5]);
  bgm.Play();
  
  level_file >> player_start.x;
  level_file >> player_start.y;
  level_file >> player_start.z;
  player_glider.SetPosition(player_start);
  
  direction_start.x = 0.0f;
  level_file >> direction_start.y;
  direction_start.y /= 360.0f;
  direction_start.z = 0.0f;
  player_glider.SetDirection(direction_start); 
  
  level_file >> laps;
  
  ring_list.SetLaps(laps);
  total_laps = laps;
  
  level_file >> num_loops;
  
  for(int i = 0; i < num_loops; i++)
  {
    float x, y, z, dir;
    
    level_file >> x;
    level_file >> y;
    level_file >> z;
    level_file >> dir;
    test_ring.SetXYZDirection(x, y, z, dir);
    ring_list.AddRing(test_ring);
  }
  main_font.Load("data/font.ttf", 20, 24, ol::Rgba::WHITE, 0);
  big_font.Load("data/font.ttf", 60, 60, ol::Rgba::WHITE, 0);
  boost_meter_texture.Load("data/boost_meter.png");
  boost_inside_texture.Load("data/boost_inside.png");
  
  boost = 1.0f;
  boosting = false;
  fade_out_alpha = 0.0f;
  timer = 0.0f;
  
  level_complete = false;
  toggle_help = false;
}

bool MAIN_GAME_LOOP::Logic(void)
{
  CheckInput();
  DoLogic();
  DrawScene();
  
  return !user_quit;
}

void MAIN_GAME_LOOP::Shutdown(void)
{
  hmap.Destroy();
  tex3d.Destroy();
  ring_list.Destroy();
  main_font.Destroy();
  boost_meter_texture.Destroy();
  boost_inside_texture.Destroy();
  bgm.Destroy();
}

void MAIN_GAME_LOOP::CheckInput(void)
{
  user_quit = false;
  
  if(KeyPress(KEY_ENTER))
  {
    toggle_help = !toggle_help;
  }
  
  if(KeyDown(KEY_A) || KeyDown(KEY_LEFT))
    player_glider.TurnLeft(true);
  else
    player_glider.TurnLeft(false);
    
  if(KeyDown(KEY_D) || KeyDown(KEY_RIGHT))
    player_glider.TurnRight(true);
  else
    player_glider.TurnRight(false);
    
  if(KeyDown(KEY_S) || KeyDown(KEY_DOWN))
    player_glider.TiltUp(true);
  else
    player_glider.TiltUp(false);
    
  if(KeyDown(KEY_W) || KeyDown(KEY_UP))
    player_glider.TiltDown(true);
  else
    player_glider.TiltDown(false);
    
  
  if(KeyPress(KEY_LSHIFT) || KeyPress(KEY_LCONTROL))
  {
    engines_sound.Stop();
    engines_sound.PlayRepeat();
  }
  
  if(KeyDown(KEY_LSHIFT) || KeyDown(KEY_LCONTROL))
  {
    player_glider.ThrusterOn(true);
  }
  else
  {
    engines_sound.Stop();
    player_glider.ThrusterOn(false);
  }
    
  if(KeyDown(KEY_SPACE) || KeyDown(KEY_ALT))
  {
    if(boost >= 1.0f || boosting == true)
    {
      if(boosting == false)
      {
        boost_sound.Play();
      }
      boosting = true;
      player_glider.UseBoost();
      boost-= 8.0f * frame_time;
      if(boost <= 0.00f)
      {
        boost = 0.0f;
        boosting = false;
      }
    }
  }
  else
  {
    boosting = false;
  }
  
  if(KeyDown(KEY_ESC))
  {
    user_quit = true;
  }
}

void MAIN_GAME_LOOP::DoLogic(void)
{
  VECTOR3D next_player_pos;
  VECTOR3D current_player_pos;
  
  player_glider.UpdateDirection();
  player_glider.GetNextPosition(next_player_pos);
  if(next_player_pos.y <= hmap.CollisionHeight(next_player_pos.x, next_player_pos.z))
  {
    player_glider.GetPosition(current_player_pos);
    //if(current_player_pos.y > next_player_pos.y)
    //{
      next_player_pos.y = current_player_pos.y + 80.0f * frame_time;
      
      player_glider.ResetYVelocity();
    //}
    next_player_pos.x = current_player_pos.x;
    next_player_pos.z = current_player_pos.z;
    //Try moving in two dimentiuons.
    /*if(next_player_pos.y > hmap.CollisionHeight(next_player_pos.x, current_player_pos.z))
    {
      next_player_pos.z = current_player_pos.z;
    }
    else if(next_player_pos.y > hmap.CollisionHeight(current_player_pos.x, next_player_pos.z))
    {
      next_player_pos.x = current_player_pos.x;
    }
    else
    {
      next_player_pos.x = current_player_pos.x;
      next_player_pos.y = current_player_pos.y;
    }*/
    //player_glider.ResetVelocity();
  }
  if(next_player_pos.y >= 200.0f)
  {
    next_player_pos.y = 200.0f;
  }
  player_glider.SetPosition(next_player_pos);
  
  if(boost < 1.0f)
  {
    boost += 0.60f * frame_time;
    if(boost > 1.0f)
    {
      boost = 1.0f;
    }
  }
  
  ring_list.CheckCollisionWithTarget(next_player_pos);
  
  if(ring_list.GotThenAll())
  {
    level_complete = true;
    if(fade_out_alpha == 0.0f)
    {
      bgm.Stop();
      
      if((int)timer <= time_tb)
      {
        clear_music.Play();
      }
      else
      {
        lose_music.Play();
      }
    }
    fade_out_alpha += 0.2 * frame_time;
  
    if(fade_out_alpha >= 1.0f)
    {
      user_quit = true;
    }
  }
  else
  {
    timer += 1000 * frame_time;
  }
}

void MAIN_GAME_LOOP::DrawScene(void)
{
  VECTOR3D camera;
  VECTOR3D glider_pos;
  
  Init3DDrawingMode();
  player_glider.GetCameraPosition(camera);
  player_glider.GetPosition(glider_pos);
  gluLookAt(camera.x, camera.y, camera.z, glider_pos.x, glider_pos.y, glider_pos.z, 0.0f, 1.0f, 0.0f);
  glDisable(GL_DEPTH_TEST);
  glDisable(GL_LIGHTING);
  glPushMatrix();
  glTranslatef(camera.x, camera.y, camera.z);
  glScalef(10.0f, 10.0f, 10.0f);
  glDisable(GL_TEXTURE_3D_EXT); 
  glEnable(GL_TEXTURE_2D);
  sky_mesh.Render();
  glPopMatrix();
  glEnable(GL_LIGHTING);
  glEnable(GL_DEPTH_TEST);
  
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_TEXTURE_3D_EXT); 
  hmap.Draw();
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_TEXTURE_3D_EXT); 
  
  player_glider.Draw();
  
  ring_list.DrawRings();
  
  Init2DDrawingMode();
  
  DrawGUI();
}

void MAIN_GAME_LOOP::DrawGUI(void)
{
  int ms, sec, min;
  float meter_alpha;
  char speed_text[50];
  VECTOR3D glider_pos;
  
  player_glider.GetPosition(glider_pos);
  
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  if(boost < 1.0f)
  {
    meter_alpha = 0.5f;
  }
  else
  {
    meter_alpha = 1.0f;
  }
  
  boost_inside_texture.Blit(7.0f, 7.0f, ol::Clipped(0.0f, 0.0f, 169.0f * boost, 17.0f), meter_alpha);
  boost_meter_texture.Blit(5.0f, 5.0f, meter_alpha);
  
  sprintf(speed_text, "Speed: %ikph", (int)(player_glider.GetSpeed() / 2.0f)); 
  main_font.SetColor(ol::Rgba( 1.0, 1.0, 1.0 )); 
  main_font.Print(speed_text, 635, 20);
  
  if(ring_list.GetLapsLeft() > 0)
  {
    sprintf(speed_text, "Lap: %i/%i", total_laps + 1 - ring_list.GetLapsLeft(), total_laps); 
  }
  else
  {
    sprintf(speed_text, "Lap: %i/%i", total_laps, total_laps); 
  }
  
  main_font.SetColor(ol::Rgba( 1.0, 1.0, 1.0 )); 
  main_font.Print(speed_text, 635, 40);
  
  if(boost >= 1.0f)
  {
    main_font.SetColor(ol::Rgba( 1.0, 0.5, 0.5 )); 
    main_font.Print("Ready!", 60, 23);
  }
  
  ConvertTimer(ms, sec, min);
  
  sprintf(speed_text, "%i:%i:%i", min, sec, ms);
  main_font.SetColor(ol::Rgba( 1.0, 1.0, 1.0 )); 
  main_font.Print(speed_text, 5, 595);
  
  if(toggle_help)
  {
    sprintf(speed_text, "X: %i Y: %i Z: %i", (int)glider_pos.x, (int)glider_pos.y, (int)glider_pos.z);
    main_font.SetColor(ol::Rgba( 1.0, 1.0, 1.0 )); 
    main_font.Print(speed_text, 5, 300);
    main_font.Print("Movement - WSAD or Arrow Keys\nThrust - Shift or Left CTRL\nBoost - Spacebar or Alt\nObjective - Fly through the active loop.", 5, 320);
  }
  
  if(fade_out_alpha != 0.0f)
  {
    
    ol::Rect rectange(0.0f, 0.0f, 0.0f, 0.0f);
  
    rectange.Draw(ol::Rgba(0.0f, 0.0f, 0.0f, 0.0f));
  
    glBegin(GL_QUADS);
  
    //fade_out_alpha = 0.75f;
  
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
    glDisable(GL_TEXTURE_2D);
    glColor4f(0.0f, 0.0f, 0.0f, fade_out_alpha);
    glVertex3f(-400.0f, 300.0f, 0.0f);
    glVertex3f(-400.0f, -300.0f, 0.0f);
    glVertex3f(400.0f, -300.0f, 0.0f);
    glVertex3f(400.0f, 300.0f, 0.0f);
  
    glEnd();
    
    big_font.Print("Stage Clear!", 400 - big_font.FirstLineWidth("Stage Clear!") / 2, 100);
  }
}

void MAIN_GAME_LOOP::SetLevelFilename(char *filename)
{
  level_filename = filename;
}

void MAIN_GAME_LOOP::ConvertTimer(int &ms, int &sec, int &min)
{
  int total_milliseconds;
  
  total_milliseconds = (int)timer;
  
  ms = total_milliseconds % 1000;
  total_milliseconds /= 1000;
  sec = total_milliseconds % 60;
  min = total_milliseconds / 60;
}

int MAIN_GAME_LOOP::GetFinalTime(void)
{
  return (int)timer;
}

void MAIN_GAME_LOOP::SetTTB(int ttb)
{
  time_tb = ttb;
}

bool MAIN_GAME_LOOP::LevelComplete(void)
{
  return level_complete;
}
