#include <OpenLayer.hpp>
#include "mesh.h"
#include "resources.h"

MESH::MESH()
{
  tex_ptr = NULL;
  vertex_data = NULL;
  num_vertices = 0;
}

MESH::~MESH()
{
  Destroy();
}

void MESH::Load(const char *file)
{
  FILE *load_me;
  
  //Open the file and make sure it exists.
  load_me = fopen(file, "r");
  if(load_me == NULL)
  {
    return;
  }
  
  //First, read the first integer that describes the number of vertices.
  fscanf(load_me, "%i\n", &num_vertices);
  
  //If we are reloading a new mesh, we should delete the old data.
  Destroy();
  
  //Now create a new list of vertex data.  There are 2 UV coordinates, 3
  //normals (x, y, z), and 3 coordinates (x, y, z) = 8 piecs of data per vertex.
  vertex_data = new float[num_vertices * 8];
  
  //Now let's read in the rest of the data.
  for(int i = 0; i < num_vertices * 8; i+=8)
  {
    fscanf(load_me, "%f %f %f %f %f %f %f %f\n", &vertex_data[i], 
          &vertex_data[i + 1], &vertex_data[i + 2], &vertex_data[i + 3],
          &vertex_data[i + 4], &vertex_data[i + 5], &vertex_data[i + 6],
          &vertex_data[i + 7]);
  }
  
  fclose(load_me);
}

void MESH::SetTexture(TEXTURE *tex)
{
  tex_ptr = tex;
}

void MESH::Render(void)
{
  glActiveTextureARB(GL_TEXTURE0_ARB);
  glEnable(GL_TEXTURE_2D);
  //Bind our texture to be using.
  if(tex_ptr != NULL)
  {
    tex_ptr->Bind();
  }
  
  /*glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D);
  //glDisable(GL_TEXTURE_2D);
  sky_texture.Bind();
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);*/
  
  //Later on, I can add some better color functions.
  glColor4ub(255, 255, 255, 255);
  
  //Enable sphere mapping if asked.
  /*  glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
    glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
    glEnable(GL_TEXTURE_GEN_S); 
    glEnable(GL_TEXTURE_GEN_T); */
  
  //Set our blend function.  Later on I can add additive blending.
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glBlendFunc(GL_ONE, GL_ONE);
  //glEnable(GL_BLEND);
  
  //Set up the array, and draw our object.
  glInterleavedArrays(GL_T2F_N3F_V3F, 0, vertex_data);
  glDrawArrays(GL_TRIANGLES, 0, num_vertices);
  
  //Disable Sphere Mapping here on out!
  //glDisable(GL_TEXTURE_GEN_S); 
  //glDisable(GL_TEXTURE_GEN_T);
  //glDisable(GL_TEXTURE_2D);
  //glDisable(GL_BLEND);
  glActiveTextureARB(GL_TEXTURE0_ARB);
}

void MESH::Destroy(void)
{
  if(vertex_data != NULL)
  {
    delete[] vertex_data;
    num_vertices = 0;
  }
}

