#include <OpenLayer.hpp>
#include <GL/glut.h>
#include "graphics.h"
#include "resources.h"
#include "ring.h"

RING::RING(void)
{
  position.x = 0.0f;
  position.y = 0.0f;
  position.z = 0.0f;
  
  y_rotation = 0.0f;
  z_rotation = 0.0f;
  target = false;
}

void RING::SetXYZDirection(float x, float y, float z, float direction)
{
  position.x = x;
  position.y = y;
  position.z = z;
  
  y_rotation = direction;
  z_rotation = 0.0f;
}

void RING::Draw(void)
{
  glPushMatrix();
  
  metal_texture.Bind();
  
  glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glEnable(GL_TEXTURE_GEN_S); 
  glEnable(GL_TEXTURE_GEN_T);
  glEnable(GL_TEXTURE_2D);
  
  glTranslatef(position.x, position.y, position.z);
  glRotatef(y_rotation, 0.0f, 1.0f, 0.0f);
  glRotatef(z_rotation, 0.0f, 0.0f, 1.0f);
  
  glutSolidTorus(3.0f, 25.0f, 8, 16);
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(position.x, position.y, position.z);
  glRotatef(y_rotation, 0.0f, 1.0f, 0.0f);
  glRotatef(-1.0f * z_rotation, 0.0f, 0.0f, 1.0f);
  
  glutWireTorus(4.0f, 25.0f, 8, 16);
  
  z_rotation += 45.0f * frame_time;
  if(z_rotation > 360.0f)
  {
    z_rotation -= 360.0f;
  }
  
  glDisable(GL_TEXTURE_GEN_S); 
  glDisable(GL_TEXTURE_GEN_T);
  
  glPopMatrix();
  
  if(target)
  {
    glDisable(GL_CULL_FACE);
    glDisable(GL_LIGHTING);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glDepthMask(GL_FALSE);
    
    glPushMatrix();
    
    glTranslatef(position.x, position.y, position.z);
    glRotatef(y_rotation, 0.0f, 1.0f, 0.0f);
    glRotatef(-1.0f * z_rotation, 0.0f, 0.0f, 1.0f);
    
    sparkle_texture.Bind();
    
    glBegin(GL_QUADS);
    
    glColor4f(0.7f, 0.7f, 1.0f, 1.0f);
    
    glTexCoord2f(0.0f, 1.0f);
    glVertex3f(-20.0f, 20.0f, 0.0f);
    glTexCoord2f(0.0f, 0.0f);
    glVertex3f(-20.0f, -20.0f, 0.0f);
    glTexCoord2f(1.0f, 0.0f);
    glVertex3f(20.0f, -20.0f, 0.0f);
    glTexCoord2f(1.0f, 1.0f);
    glVertex3f(20.0f, 20.0f, 0.0f);
    
    glEnd();
    
    glPopMatrix();
    
    glPushMatrix();
    
    glTranslatef(position.x, position.y, position.z);
    glRotatef(y_rotation, 0.0f, 1.0f, 0.0f);
    glRotatef(1.0f * z_rotation, 0.0f, 0.0f, 1.0f);
    
    sparkle_texture.Bind();
    
    glBegin(GL_QUADS);
    
    glColor4f(0.7f, 0.7f, 1.0f, 1.0f);
    
    glTexCoord2f(0.0f, 1.0f);
    glVertex3f(-20.0f, 20.0f, 0.0f);
    glTexCoord2f(0.0f, 0.0f);
    glVertex3f(-20.0f, -20.0f, 0.0f);
    glTexCoord2f(1.0f, 0.0f);
    glVertex3f(20.0f, -20.0f, 0.0f);
    glTexCoord2f(1.0f, 1.0f);
    glVertex3f(20.0f, 20.0f, 0.0f);
    
    glEnd();
    
    glPopMatrix();
    
    glEnable(GL_CULL_FACE);
    glEnable(GL_LIGHTING);
    glDisable(GL_BLEND);
    glDepthMask(GL_TRUE);
    //glEnable(GL_DEPTH_TEST);
  }
}

void RING::SetTarget(bool is_target)
{
  target = is_target;
}

bool RING::CheckPointCollision(VECTOR3D &ship_pos)
{
  VECTOR3D distance;
  
  distance.x = ship_pos.x - position.x;
  distance.y = ship_pos.y - position.y;
  distance.z = ship_pos.z - position.z;
  
  if(distance.GetLength() < 22.0f)
  {
    return true;
  }
  
  return false;
}
