#include <GL/glew.h>
#include <allegro.h>
#include "texture3d.h"

TEXTURE3D::TEXTURE3D()
{
  tex_num = 0;
}

TEXTURE3D::~TEXTURE3D()
{
  Destroy();
}

void TEXTURE3D::Load(const char *lowest_file, const char *mid_l, 
            const char *mid_h, const char *highest)
{
  BITMAP *low, *mid_low, *mid_high, *high;
  unsigned char *rgb_tex;
  int height, width;
  int pixel;
  
  glewInit();

  if(tex_num != 0)
  {
    Destroy();
  }
    
  low = load_bitmap(lowest_file, NULL);
  mid_low = load_bitmap(mid_l, NULL);
  mid_high = load_bitmap(mid_h, NULL);
  high = load_bitmap(highest, NULL);
  
  width = low->w;
  height = low->h;
  rgb_tex = new unsigned char[4 * height * width * 3];
  
  for(int y = 0; y < height; y++)
  {
    for(int x = 0; x < width; x++)
    {
      pixel = getpixel(low, x, y);
      rgb_tex[(0 * width * height * 3) + (y * width * 3) + (x * 3)] = (unsigned char) getr(pixel);
      rgb_tex[(0 * width * height * 3) + (y * width * 3) + (x * 3) + 1] = (unsigned char) getg(pixel);
      rgb_tex[(0 * width * height * 3) + (y * width * 3) + (x * 3) + 2] = (unsigned char) getb(pixel);
      
      pixel = getpixel(mid_low, x, y);
      rgb_tex[(1 * width * height * 3) + (y * width * 3) + (x * 3)] = (unsigned char) getr(pixel);
      rgb_tex[(1 * width * height * 3) + (y * width * 3) + (x * 3) + 1] = (unsigned char) getg(pixel);
      rgb_tex[(1 * width * height * 3) + (y * width * 3) + (x * 3) + 2] = (unsigned char) getb(pixel);
      
      pixel = getpixel(mid_high, x, y);
      rgb_tex[(2 * width * height * 3) + (y * width * 3) + (x * 3)] = (unsigned char) getr(pixel);
      rgb_tex[(2 * width * height * 3) + (y * width * 3) + (x * 3) + 1] = (unsigned char) getg(pixel);
      rgb_tex[(2 * width * height * 3) + (y * width * 3) + (x * 3) + 2] = (unsigned char) getb(pixel);
      
      pixel = getpixel(high, x, y);
      rgb_tex[(3 * width * height * 3) + (y * width * 3) + (x * 3)] = (unsigned char) getr(pixel);
      rgb_tex[(3 * width * height * 3) + (y * width * 3) + (x * 3) + 1] = (unsigned char) getg(pixel);
      rgb_tex[(3 * width * height * 3) + (y * width * 3) + (x * 3) + 2] = (unsigned char) getb(pixel);
    }
  }
  
  glMatrixMode(GL_TEXTURE);
  
  glEnable(GL_TEXTURE_3D_EXT);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  
  glGenTextures(1, &tex_num);
  glBindTexture(GL_TEXTURE_3D_EXT, tex_num);
  glTexParameteri(GL_TEXTURE_3D_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_3D_EXT, GL_TEXTURE_WRAP_R_EXT, GL_REPEAT);
  glTexImage3DEXT(GL_TEXTURE_3D_EXT, 0, 4, width, height, 4, 0,
                    GL_RGB, GL_UNSIGNED_BYTE, rgb_tex);
  
  destroy_bitmap(low);
  destroy_bitmap(mid_low);
  destroy_bitmap(mid_high);
  destroy_bitmap(high);
  
  delete[] rgb_tex;
}

void TEXTURE3D::Destroy()
{
  glDeleteTextures(1,	&tex_num);
  tex_num = 0;
}

void TEXTURE3D::Bind()
{
  glBindTexture(GL_TEXTURE_3D_EXT, tex_num);
}
