#include "title_loop.h"
#include "graphics.h"
#include "input.h"
#include "resources.h"

#include <GL/glut.h>

#define NG_X 3.7f
#define NG_Y -3.0f
#define E_X 1.8f
#define E_Y -4.3f

void TITLE_LOOP::Init(void)
{
  title_font.Load("data/font.ttf", 100, 100, ol::Rgba::WHITE, 0);
  selection_font.Load("data/font.ttf", 32, 32, ol::Rgba::WHITE, 0);
  bg_rotation = 0.0f;
  select_rotation = 0.0f;
  current_selection = 0;
}

bool TITLE_LOOP::Logic(void)
{
  float select_x, select_y;
  
  switch(current_selection)
  {
    case 0:
      select_x = NG_X;
      select_y = NG_Y;
      break;
    default:
      select_x = E_X;
      select_y = E_Y;
      break;
  }
  
  Init3DDrawingMode();
  
  glPushMatrix();
  
  glDisable(GL_TEXTURE_2D);
  glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
  
  glTranslatef(20.0f, 0.0f, -20.0f);
  glRotatef(bg_rotation, 0.0f, 1.0f, 0.0f);
  glRotatef(90.0, 1.0f, 0.0f, 0.0f);
  
  glutWireSphere(30.0f, 40, 40);
  
  bg_rotation += 30.0f * frame_time;
  
  glPopMatrix();
  
  glEnable(GL_TEXTURE_2D);
  //glDisable(GL_LIGHTING);
  glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
  glEnable(GL_TEXTURE_GEN_S); 
  glEnable(GL_TEXTURE_GEN_T);
  
  glPushMatrix();
  
  glTranslatef(-1.0f * select_x, select_y, -10.0f);
  glRotatef(select_rotation, 1.0f, 0.0f, 0.0f);
  
  select_mesh.Render();
  
  glPopMatrix();
  
  glPushMatrix();
  
  glTranslatef(select_x, select_y, -10.0f);
  glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
  glRotatef(-1.0f * select_rotation, 1.0f, 0.0f, 0.0f);
  
  select_mesh.Render();
  
  glPopMatrix();
  
  glDisable(GL_TEXTURE_GEN_S); 
  glDisable(GL_TEXTURE_GEN_T);
  
  select_rotation += 90.0f * frame_time;
  
  Init2DDrawingMode();
  
  //title_font.Print("G-Loop", 400 - title_font.FirstLineWidth("G-Loop") / 2, 200);
  //selection_font.Print("New Game", 400 - selection_font.FirstLineWidth("New Game") / 2, 400);
  //selection_font.Print("Exit", 400 - selection_font.FirstLineWidth("Exit") / 2, 440);
  
  title_font.Print("G-Loop", 200, 200);
  selection_font.Print("New Game", 400 - selection_font.FirstLineWidth("New Game") / 2, 400);
  selection_font.Print("Exit", 400 - selection_font.FirstLineWidth("Exit") / 2, 440);
  
  if(KeyPress(KEY_UP) || KeyPress(KEY_W))
  {
    current_selection = 0;
    click_sound.Play();
  }
  
  if(KeyPress(KEY_DOWN) || KeyPress(KEY_S))
  {
    current_selection = 1;
    click_sound.Play();
  }
  
  if(KeyPress(KEY_ENTER) || KeyPress(KEY_ESC) || KeyPress(KEY_LCONTROL))
  {
    if(KeyDown(KEY_ESC))
    {
      current_selection = 1;
    }
    else
    {
      select_sound.Play();
    }
    return false;
  }
  return true;
}

bool TITLE_LOOP::SelectQuit(void)
{
  if(current_selection == 1)
  {
    return true;
  }
  
  return false;
}

void TITLE_LOOP::Shutdown(void)
{
  title_font.Destroy();
  selection_font.Destroy();
}
